#include <allegro.h>

#include <stdio.h>
#include <unistd.h>
#include <sys/poll.h>
#include <string.h>


void print(const char *str)
{
	textout_ex (screen, font, str, 0, 230, 15, -1);
	blit (screen, screen, 0, 10, 0, 0, 320, 230);
	rectfill (screen, 0, 230, 319, 239, 0);
}


volatile int time_in_secs = 0;

void inc_time_in_secs (void)
{
	++time_in_secs;
}


const char *poem[] = {
	"Mary had a little lamb\n",
	"Its fleece was white as snow\n",
	"And everywhere that Mary went\n",
	"The lamb was sure to go.\n",
	NULL
};


int main (void)
{
	allegro_init();
	install_joystick(JOY_TYPE_AUTODETECT);
	install_timer();
	install_int (inc_time_in_secs, 1000);
	
	set_gfx_mode (GFX_AUTODETECT, 320, 240, 0, 0);
	set_palette (default_palette);
	clear_to_color(screen, 0);
	
	print("USB serial test program");
	
	print("Opening /dev/ttygs0...");
	
	int fd = open("/dev/ttygs0", O_RDWR);
	if (fd == -1) {
		print("failed");
	} else {
		print("ok");
	}
	
	enum { BUF_SIZE = 1024 };
	char buffer[BUF_SIZE];
	int pos = 0;
	int old_time = time_in_secs;
	bool go = false;

	do {
		if (fd >= 0) {
			
			/* Input */
			if (pos < BUF_SIZE) {
				struct pollfd pollfds[1];
				pollfds[0].fd = fd;
				pollfds[0].events = POLLIN;
				pollfds[0].revents = 0;
				if (poll (pollfds, 1, 0) > 0) {
					int count = read (fd, buffer + pos, 1);
					if (count < 0) {
						print("read error");
					} else if (count > 0) {
						char buf[1024];
						sprintf (buf, "read 0x%02x (%c)", buffer[0], (buffer[0] >= 0x20 && buffer[0] <= 0x7e) ? buffer[0] : '.');
						print(buf);
						go = true;
					}
				}
			}

			/* Output */
			if (pos > 0) {
				struct pollfd pollfds[1];
				pollfds[0].fd = fd;
				pollfds[0].events = POLLOUT;
				pollfds[0].revents = 0;
				if (poll (pollfds, 1, 0) > 0) {
					int count = write (fd, buffer, pos);
					if (count < 0) {
						print("write error");
					} else {
						memmove (buffer, buffer + count, pos - count);
						pos -= count;
					}
				}
			}

			/* More output */
			if (time_in_secs != old_time) {
				old_time = time_in_secs;
				struct pollfd pollfds[1];
				pollfds[0].fd = fd;
				pollfds[0].events = POLLOUT;
				pollfds[0].revents = 0;
				if (poll (pollfds, 1, 0) > 0) {
					int count = write (fd, poem[old_time%4], strlen(poem[old_time%4]));
					if (count < 0) {
						print("write error");
					}
				}
			}
		}

		poll_joystick();

	} while (!joy[0].button[1].b);
	
	print("Closing /dev/ttygs0...");
	
	close(fd);
	
	print("Exiting");

	return 0;
}
END_OF_MAIN()

